'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var child_process = require('child_process');
var chalk = require('chalk');
var spawn = require('cross-spawn');
var open = require('open');
var fs = require('fs');
var path = require('path');
var isGlob = require('is-glob');
var globSync = require('tiny-glob/sync');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var chalk__default = /*#__PURE__*/_interopDefaultLegacy(chalk);
var spawn__default = /*#__PURE__*/_interopDefaultLegacy(spawn);
var open__default = /*#__PURE__*/_interopDefaultLegacy(open);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var isGlob__default = /*#__PURE__*/_interopDefaultLegacy(isGlob);
var globSync__default = /*#__PURE__*/_interopDefaultLegacy(globSync);

// based on https://github.com/facebook/create-react-app/blob/master/packages/react-dev-utils/openBrowser.js
// https://github.com/sindresorhus/open#app
var OSX_CHROME = 'google chrome';
var Action;
(function (Action) {
    Action[Action["NONE"] = 0] = "NONE";
    Action[Action["BROWSER"] = 1] = "BROWSER";
    Action[Action["SCRIPT"] = 2] = "SCRIPT";
})(Action || (Action = {}));
function getBrowserEnv() {
    // Attempt to honor this environment variable.
    // It is specific to the operating system.
    // See https://github.com/sindresorhus/open#app for documentation.
    var value = process.env.BROWSER;
    var args = process.env.BROWSER_ARGS
        ? process.env.BROWSER_ARGS.split(' ')
        : [];
    var action;
    if (!value) {
        // Default.
        action = Action.BROWSER;
    }
    else if (value.toLowerCase().endsWith('.js')) {
        action = Action.SCRIPT;
    }
    else if (value.toLowerCase() === 'none') {
        action = Action.NONE;
    }
    else {
        action = Action.BROWSER;
    }
    return { action: action, value: value, args: args };
}
function executeNodeScript(scriptPath, url) {
    var extraArgs = process.argv.slice(2);
    var child = spawn__default['default']('node', tslib.__spreadArray(tslib.__spreadArray([scriptPath], extraArgs), [url]), {
        stdio: 'inherit',
    });
    child.on('close', function (code) {
        if (code !== 0) {
            console.log();
            console.log(chalk__default['default'].red('The script specified as BROWSER environment variable failed.'));
            console.log(chalk__default['default'].cyan(scriptPath) + " exited with code " + code + ".\n");
        }
    });
    return true;
}
function startBrowserProcess(browser, url, args) {
    // If we're on OS X, the user hasn't specifically
    // requested a different browser, we can try opening
    // Chrome with AppleScript. This lets us reuse an
    // existing tab when possible instead of creating a new one.
    var shouldTryOpenChromeWithAppleScript = process.platform === 'darwin' &&
        (typeof browser !== 'string' || browser === OSX_CHROME);
    if (shouldTryOpenChromeWithAppleScript) {
        try {
            // Try our best to reuse existing tab
            // on OS X Google Chrome with AppleScript
            child_process.execSync('ps cax | grep "Google Chrome"');
            child_process.execSync('osascript ../openChrome.applescript "' + encodeURI(url) + '"', {
                cwd: __dirname,
                stdio: 'ignore',
            });
            return true;
        }
        catch (_a) {
            // Ignore errors.
        }
    }
    // Another special case: on OS X, check if BROWSER has been set to "open".
    // In this case, instead of passing `open` to `opn` (which won't work),
    // just ignore it (thus ensuring the intended behavior, i.e. opening the system browser):
    // https://github.com/facebook/create-react-app/pull/1690#issuecomment-283518768
    if (process.platform === 'darwin' && browser === 'open') {
        browser = undefined;
    }
    // If there are arguments, they must be passed as array with the browser
    if (typeof browser === 'string' && args.length > 0) {
        browser = tslib.__spreadArray([browser], args);
    }
    // Fallback to open
    // (It will always open new tab)
    // eslint-disable-next-line sonar/no-try-promise
    try {
        var options = { app: browser, wait: false };
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        open__default['default'](url, options).catch(function () { }); // Prevent `unhandledRejection` error.
        return true;
    }
    catch (_b) {
        return false;
    }
}
/**
 * Reads the BROWSER environment variable and decides what to do with it. Returns
 * true if it opened a browser or ran a node.js script, otherwise false.
 */
function openBrowser(url) {
    var _a = getBrowserEnv(), action = _a.action, value = _a.value, args = _a.args;
    switch (action) {
        case Action.NONE:
            // Special case: BROWSER="none" will prevent opening completely.
            return false;
        case Action.SCRIPT:
            return executeNodeScript(value, url);
        case Action.BROWSER:
            return startBrowserProcess(value, url, args);
        default:
            throw new Error('Not implemented.');
    }
}

var _a;
var DEV = 'development';
var PROD = 'production';
var NODE_ENV = (_a = process.env.NODE_ENV, _a === void 0 ? DEV : _a);
var __DEV__ = NODE_ENV === DEV;
var __PROD__ = NODE_ENV === PROD;
var NODE_MODULES_REG = /[/\\]node_modules[/\\]/;
var CWD = process.cwd();
// eslint-disable-next-line node/no-deprecated-api, sonar/deprecation
var EXTENSIONS = tslib.__spreadArray(['.ts', '.tsx'], Object.keys(require.extensions));

var tryPkg = function (pkg) {
    try {
        return require.resolve(pkg);
    }
    catch (_a) { }
};
var tryRequirePkg = function (pkg) {
    try {
        // eslint-disable-next-line @typescript-eslint/no-require-imports, @typescript-eslint/no-unsafe-return
        return require(pkg);
    }
    catch (_a) { }
};
var isPkgAvailable = function (pkg) { return !!tryPkg(pkg); };
var isTsAvailable = isPkgAvailable('typescript');
var isAngularAvailable = isPkgAvailable('@angular/core');
var isMdxAvailable = isPkgAvailable('@mdx/mdx') || isPkgAvailable('@mdx/react');
var isReactAvailable = isPkgAvailable('react');
var isSvelteAvailable = isPkgAvailable('svelte');
var isVueAvailable = isPkgAvailable('vue');
var tryFile = function (filePath, includeDir) {
    if (includeDir === void 0) { includeDir = false; }
    if (typeof filePath === 'string') {
        return fs__default['default'].existsSync(filePath) &&
            (includeDir || fs__default['default'].statSync(filePath).isFile())
            ? filePath
            : '';
    }
    for (var _i = 0, _a = filePath || []; _i < _a.length; _i++) {
        var file = _a[_i];
        if (tryFile(file, includeDir)) {
            return file;
        }
    }
    return '';
};
var tryExtensions = function (filepath, extensions) {
    if (extensions === void 0) { extensions = EXTENSIONS; }
    var ext = tslib.__spreadArray(tslib.__spreadArray([], extensions), ['']).find(function (ext) { return tryFile(filepath + ext); });
    return ext == null ? '' : filepath + ext;
};
var tryGlob = function (paths, options) {
    if (options === void 0) { options = {}; }
    var _a = typeof options === 'string' ? { baseDir: options } : options, _b = _a.absolute, absolute = _b === void 0 ? true : _b, _c = _a.baseDir, baseDir = _c === void 0 ? CWD : _c;
    return paths.reduce(function (acc, pkg) {
        return tslib.__spreadArray(tslib.__spreadArray([], acc), (isGlob__default['default'](pkg)
            ? globSync__default['default'](pkg, {
                absolute: absolute,
                cwd: baseDir,
            })
            : [tryFile(path__default['default'].resolve(baseDir, pkg), true)])).filter(Boolean);
    }, []);
};
var identify = function (_) { return !!_; };
var findUp = function (searchEntry, searchFile) {
    if (searchFile === void 0) { searchFile = 'package.json'; }
    console.assert(path__default['default'].isAbsolute(searchEntry));
    if (!tryFile(searchEntry, true) ||
        (searchEntry !== CWD && !searchEntry.startsWith(CWD + path__default['default'].sep))) {
        return '';
    }
    searchEntry = path__default['default'].resolve(fs__default['default'].statSync(searchEntry).isDirectory()
        ? searchEntry
        : path__default['default'].resolve(searchEntry, '..'));
    do {
        var searched = tryFile(path__default['default'].resolve(searchEntry, searchFile));
        if (searched) {
            return searched;
        }
        searchEntry = path__default['default'].resolve(searchEntry, '..');
    } while (searchEntry === CWD || searchEntry.startsWith(CWD + path__default['default'].sep));
    return '';
};
var arrayify = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return args.reduce(function (arr, curr) {
        arr.push.apply(arr, (Array.isArray(curr) ? curr : curr == null ? [] : [curr]));
        return arr;
    }, []);
};

var pkg = tryRequirePkg(path__default['default'].resolve('package.json')) || {};
var lernaConfig = tryRequirePkg(path__default['default'].resolve('lerna.json')) || {};
var pkgsPath = lernaConfig.packages || pkg.workspaces || [];
var isMonorepo = Array.isArray(pkgsPath) && pkgsPath.length > 0;
var monorepoPkgs = isMonorepo ? tryGlob(pkgsPath) : [];

exports.CWD = CWD;
exports.DEV = DEV;
exports.EXTENSIONS = EXTENSIONS;
exports.NODE_ENV = NODE_ENV;
exports.NODE_MODULES_REG = NODE_MODULES_REG;
exports.PROD = PROD;
exports.__DEV__ = __DEV__;
exports.__PROD__ = __PROD__;
exports.arrayify = arrayify;
exports.findUp = findUp;
exports.identify = identify;
exports.isAngularAvailable = isAngularAvailable;
exports.isMdxAvailable = isMdxAvailable;
exports.isMonorepo = isMonorepo;
exports.isPkgAvailable = isPkgAvailable;
exports.isReactAvailable = isReactAvailable;
exports.isSvelteAvailable = isSvelteAvailable;
exports.isTsAvailable = isTsAvailable;
exports.isVueAvailable = isVueAvailable;
exports.monorepoPkgs = monorepoPkgs;
exports.openBrowser = openBrowser;
exports.tryExtensions = tryExtensions;
exports.tryFile = tryFile;
exports.tryGlob = tryGlob;
exports.tryPkg = tryPkg;
exports.tryRequirePkg = tryRequirePkg;
