"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.arrayify = exports.findUp = exports.identify = exports.tryGlob = exports.tryExtensions = exports.tryFile = exports.isVueAvailable = exports.isSvelteAvailable = exports.isReactAvailable = exports.isMdxAvailable = exports.isAngularAvailable = exports.isTsAvailable = exports.isPkgAvailable = exports.tryRequirePkg = exports.tryPkg = void 0;
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const is_glob_1 = tslib_1.__importDefault(require("is-glob"));
const sync_1 = tslib_1.__importDefault(require("tiny-glob/sync"));
const constants_1 = require("./constants");
const tryPkg = (pkg) => {
    try {
        return require.resolve(pkg);
    }
    catch (_a) { }
};
exports.tryPkg = tryPkg;
const tryRequirePkg = (pkg) => {
    try {
        // eslint-disable-next-line @typescript-eslint/no-require-imports, @typescript-eslint/no-unsafe-return
        return require(pkg);
    }
    catch (_a) { }
};
exports.tryRequirePkg = tryRequirePkg;
const isPkgAvailable = (pkg) => !!exports.tryPkg(pkg);
exports.isPkgAvailable = isPkgAvailable;
exports.isTsAvailable = exports.isPkgAvailable('typescript');
exports.isAngularAvailable = exports.isPkgAvailable('@angular/core');
exports.isMdxAvailable = exports.isPkgAvailable('@mdx/mdx') || exports.isPkgAvailable('@mdx/react');
exports.isReactAvailable = exports.isPkgAvailable('react');
exports.isSvelteAvailable = exports.isPkgAvailable('svelte');
exports.isVueAvailable = exports.isPkgAvailable('vue');
const tryFile = (filePath, includeDir = false) => {
    if (typeof filePath === 'string') {
        return fs_1.default.existsSync(filePath) &&
            (includeDir || fs_1.default.statSync(filePath).isFile())
            ? filePath
            : '';
    }
    for (const file of filePath || []) {
        if (exports.tryFile(file, includeDir)) {
            return file;
        }
    }
    return '';
};
exports.tryFile = tryFile;
const tryExtensions = (filepath, extensions = constants_1.EXTENSIONS) => {
    const ext = [...extensions, ''].find(ext => exports.tryFile(filepath + ext));
    return ext == null ? '' : filepath + ext;
};
exports.tryExtensions = tryExtensions;
const tryGlob = (paths, options = {}) => {
    const { absolute = true, baseDir = constants_1.CWD } = typeof options === 'string' ? { baseDir: options } : options;
    return paths.reduce((acc, pkg) => [
        ...acc,
        ...(is_glob_1.default(pkg)
            ? sync_1.default(pkg, {
                absolute,
                cwd: baseDir,
            })
            : [exports.tryFile(path_1.default.resolve(baseDir, pkg), true)]),
    ].filter(Boolean), []);
};
exports.tryGlob = tryGlob;
const identify = (_) => !!_;
exports.identify = identify;
const findUp = (searchEntry, searchFile = 'package.json') => {
    console.assert(path_1.default.isAbsolute(searchEntry));
    if (!exports.tryFile(searchEntry, true) ||
        (searchEntry !== constants_1.CWD && !searchEntry.startsWith(constants_1.CWD + path_1.default.sep))) {
        return '';
    }
    searchEntry = path_1.default.resolve(fs_1.default.statSync(searchEntry).isDirectory()
        ? searchEntry
        : path_1.default.resolve(searchEntry, '..'));
    do {
        const searched = exports.tryFile(path_1.default.resolve(searchEntry, searchFile));
        if (searched) {
            return searched;
        }
        searchEntry = path_1.default.resolve(searchEntry, '..');
    } while (searchEntry === constants_1.CWD || searchEntry.startsWith(constants_1.CWD + path_1.default.sep));
    return '';
};
exports.findUp = findUp;
const arrayify = (...args) => args.reduce((arr, curr) => {
    arr.push(...(Array.isArray(curr) ? curr : curr == null ? [] : [curr]));
    return arr;
}, []);
exports.arrayify = arrayify;
//# sourceMappingURL=helpers.js.map